@@ -34,6 +34,13 @@
 #'                            `simultaneous`, `semifull`, `full`, `fullreverse` and `gold`. 
 #'                            Default, `raw`.
 #'                            
+#' @param confidence A numeric value between 0 and 1 indicating that confidence intervals 
+#'                   should be estimated using all the intermediate unit tables 
+#'                   obtained by rescaling each unit. If `NULL` (default), no confidence intervals 
+#'                   are computed. Confidence interval estimation is only allowed when a single 
+#'                   weight is considered. That is, this argument is automatically set to `NULL` if 
+#'                   `emphasis` is a vector of length greater than one.
+#'
 #' @param apriori data.frame (or matrix) of order J0xK0 with an initial estimate of the
 #'                (row-standarized) global voter transition proportions/fractions, pjk0, between
 #'                the first J0 (election) options of election 1 and the first K0 (election) options
@@ -210,14 +217,17 @@
 #'  \item{HETe}{ The estimated heterogeneity index as defined in equation (15) of Pavia and Romero (2022).}
 #'  \item{VTM.complete}{ A matrix of order JxK with the estimated proportions of row-standardized vote transitions from election 1 to election 2, including in `raw`, `regular`, `ordinary` and `enriched` scenarios the row and the column corresponding to net_entries and net_exits even when they are really small, less than 1% in all units.}
 #'  \item{VTM.complete.votes}{ A matrix of order JxK with the estimated vote transitions from election 1 to election 2, including in `raw`, `regular`, `ordinary` and `enriched` scenarios the row and the column corresponding to net_entries and net_exits even when they are really small, less than 1% in all units.}
-#'  \item{VTM.prop.units}{ An array of order JxKxI with the estimated proportions of vote transitions from election 1 to election 2 attained for each unit after adjusting the **lphom()** initial estimate.}
-#'  \item{VTM.votes.units}{ An array of order JxKxI with the estimated matrix of vote transitions from election 1 to election 2 attained for each unit after adjusting the **lphom()** initial estimate.}
+#'  \item{VTM.prop.units}{ An array of order JxKxI with the estimated proportions of vote transitions from election 1 to election 2 attained for each unit corresponding to the emphasis (weight) minimum observed errors.}
+#'  \item{VTM.votes.units}{ An array of order JxKxI with the estimated matrix of vote transitions from election 1 to election 2 attained for each unit corresponding to the emphasis (weight) minimum observed errors.}
+#'  \item{VTM.confidence}{An array of dimension J×K×2 containing the (1 – `confidence`) limits of the confidence intervals for the proportions of vote transitions from election 1 to election 2, derived from the intermediate estimates. This is a `NULL` if `confidence = NULL`.}
+#'  \item{VTM.units.confidence}{An array of dimension J×K×I×2 containing the (1 – `confidence`) limits of the confidence intervals for the proportions of vote transitions from election 1 to election 2 in each unit, derived from the intermediate estimates. This is a `NULL` if `confidence = NULL`.}
+#'  \item{VTM.votes.units.intermediate}{ An array of order JxKxIxI with the intermediate estimated voter transitions from election 1 to election 2 attained in each unit with each rescaled unit. This is a `NULL` array if `confidence = NULL`.}
 #'  \item{VTM.sequence}{ Array of order JxKxlength(emphasis) with the global estimated matrices corresponding to each weight.}
 #'  \item{zeros}{ A list of vectors of length two, indicating the election options for which no transfer of votes are allowed between election 1 and election 2.}
 #'  \item{errors}{ A matrix of order Ixlength(emphasis) with the expected errors for each unit and weight. The
 #'                solution determined by `VTM.prop.units` or `VTM.votes.units` is the one obtained combining the unit solutions
 #'                corresponding to the minimum observed errors.}
-#'  \item{VTM.prop.units.by.emphasis}{ An array of order JxKxIxlength(emphasis) with the estimated proportions of vote transitions from election 1 to election 2 attained in each unit for each weight.
+#'  \item{VTM.prop.units.by.emphasis}{ An array of order JxKxIxlength(emphasis) with the final estimated proportions of vote transitions from election 1 to election 2 attained in each unit for each weight.
 #'                                    This is a `NULL` array if `save.local.by.emphasis = FALSE`.}
 #'  \item{deterministic.bounds}{ A list of two matrices of order JxK and two arrays of order JxKxI containing for each vote transition the lower and upper allowed proportions given the observed aggregates.}
 #'  \item{inputs}{ A list containing all the objects with the values used as arguments by the function.}
@@ -255,6 +265,7 @@ rslphom <- function(votes_election1,
                     new_and_exit_voters = c("raw", "regular", "ordinary", "enriched", 
                                             "adjust1", "adjust2", "simultaneous", 
                                             "semifull", "full", "fullreverse", "gold"),
+                    confidence = NULL,
                     apriori = NULL,
                     lambda = 0.5,
                     uniform = TRUE,
@@ -278,6 +289,12 @@ rslphom <- function(votes_election1,
   if (max(weights) >= 1 | min(weights) < 0)
     stop('Invalid "emphasis" values. The values of "emphasis" must be smaller than one and non-negative.')
   
+  if (length(weights) > 1) confidence <- NULL
+  if (!is.null(confidence)){
+    if (confidence <= 0 | confidence >= 1){
+      stop('Invalid "confidence" value. The value of "confidence" must be in the interval (0, 1).')
+    }
+  }
   if (integers.solver == "lp_solve"){
     dec2counts <- dec2counts_lp
   } else {
@@ -307,6 +324,11 @@ rslphom <- function(votes_election1,
   VTM.prop.units.by.emphasis <- NULL
   if (save.local.by.emphasis)
     VTM.prop.units.by.emphasis <- array(NA, c(dim(lphom_inic$VTM.complete), nrow(lphom_inic$origin), n.weights))
+  if (!is.null(confidence)){
+    VTM.votes.units.intermediate <- array(NA, c(dim(lphom_inic$VTM.complete), nrow(lphom_inic$origin), nrow(lphom_inic$origin)))
+  } else {
+    VTM.votes.units.intermediate <- VTM.confidence <- VTM.units.confidence <- NULL
+  }
   
   # Local solutions
   errors.min <- rep(Inf, nrow(lphom_inic$origin))
@@ -340,6 +362,26 @@ rslphom <- function(votes_election1,
     VTM.sequence[, , w] <- HET_MT.votos_MT.prop_Y(votos_units)$MT.pro
   } # End loop weights
   
+  if (!is.null(confidence)){
+    peso <- weights
+    intermediate <- VTM.votes.units.intermediate
+    for (i in 1L:nrow(lphom_inic$origin)){
+      # Solution lphom tras reescalado de observaciones individuales
+      lphom.temp <- rescaled(lphom.object = lphom_inic, w = peso, ii = i, ceros = zeros)
+      for (s in 1L:nrow(lphom_inic$origin)){
+       intermediate[, , s, i] <- lphom_unit(lphom.object = lphom.temp, iii = s, solver = solver)
+        if (integers){
+          intermediate[, , s, i] <- dec2counts(intermediate[, , s, i]*lphom_inic$origin[s, ],
+                                               lphom_inic$origin[s,], lphom_inic$destination[s,])
+        } else {
+          intermediate[, , s, i] <- intermediate[, , s, i]/rowSums(intermediate[, , s, i])*lphom_inic$origin[s, ]
+        }
+        intermediate[lphom_inic$origin[s, ] == 0L, , s, i] <- 0L
+      } # End loop units
+    } # End loop rescaled unitsç
+    VTM.votes.units.intermediate <- intermediate
+  } # End if confidence
+  
   votos_units.sol[is.na(votos_units.sol)] <- 0L
   VTM_votos_homogeneos <- HET_MT.votos_MT.prop_Y(votos_units.sol)
   VTM_votos <- VTM_votos_homogeneos$MT.votos
@@ -369,6 +411,27 @@ rslphom <- function(votes_election1,
   inic <- lphom_inic[c(1L:6L, 11L)]
   names(inic) <- paste0(names(inic), "_init")
   
+  if (!is.null(confidence)){
+    VTM.units.confidence <- array(NA, c(dim(lphom_inic$VTM.complete), nrow(lphom_inic$origin), 2L))
+    alpha <- (1 - confidence)/2
+    VTM.units.confidence[, , , 1L] <- apply(intermediate, c(1L, 2L, 3L), stats::quantile, probs = alpha)
+    VTM.units.confidence[, , , 2L] <- apply(intermediate, c(1L, 2L, 3L), stats::quantile, probs = 1 - alpha)
+    VTM.confidence <- array(NA, c(dim(lphom_inic$VTM.complete), 2L))
+    VTM.confidence[, , 1L] <- apply(VTM.units.confidence[, , , 1L], c(1L, 2L), sum)
+    VTM.confidence[, , 2L] <- apply(VTM.units.confidence[, , , 2L], c(1L, 2L), sum)
+    VTM.confidence[, , 1L] <- pmin(VTM.complete, VTM.confidence[, , 1L]/rowSums(VTM_votos))
+    VTM.confidence[, , 2L] <- pmax(VTM.complete, pmin(1, VTM.confidence[, , 2L]/rowSums(VTM_votos)))
+    VTM.units.confidence[, , , 1L] <- pmin(VTM_units, sweep(VTM.units.confidence[, , , 1L], 
+                                                            c(1L, 3L), t(lphom_inic$origin), "/"))
+    VTM.units.confidence[, , , 2L] <- pmax(VTM_units, pmin(1L, sweep(VTM.units.confidence[, , , 2L], 
+                                            c(1L, 3L), t(lphom_inic$origin), "/")))
+    VTM.units.confidence[is.na(VTM.units.confidence)] <- 0
+    dimnames(VTM.confidence) <- c(dimnames(VTM.complete),
+                                list(c("Inferior", "Superior")))
+    dimnames(VTM.units.confidence) <- c(dimnames(VTM_units.sol),
+                                  list(c("Inferior", "Superior")))
+  }
+  
   # Caso de filas o columnas con cero votos
   filas0 <- which(rowSums(VTM_votos) == 0)
   colum0 <- which(colSums(VTM_votos) == 0)
@@ -381,7 +444,9 @@ rslphom <- function(votes_election1,
   output <- list("VTM" = VTM, "VTM.votes" = VTM.votes, "OTM" = OTM, "HETe" = HETe,
                  "VTM.complete" = VTM.complete, "VTM.complete.votes" = VTM_votos,
                  "VTM.prop.units" = VTM_units, "VTM.votes.units" = votos_units,
-                 "VTM.sequence" = VTM.sequence, "zeros" = zeros,
+                 "VTM.confidence" = VTM.confidence, "VTM.units.confidence" = VTM.units.confidence,
+                 "VTM.votes.units.intermediate" = VTM.votes.units.intermediate,
+                 "VTM.sequence" = VTM.sequence, "zeros" = zeros, 
                  "VTM.prop.units.by.emphasis" = VTM.prop.units.by.emphasis, "errors" = errors,
                  "deterministic.bounds" = det.bounds, "inputs" = inputs, "origin" = lphom_inic$origin,
                  "destination" = lphom_inic$destination, "EHet" = EHet, "solution_init" = inic)
